/*
 * Snake.c
 *
 * Created: 17.05.2018 14:20:53
 * Author : Ana
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

#define UP 1
#define LEFT 2
#define RIGHT 3
#define DOWN 4

#define INIT_LEN 3
#define INIT_X 40
#define INIT_Y 40
#define EASY 0
#define HARD 1
#define DIM 2

int points = 0;
int lives = 3;
int mod;

typedef struct {
	char x;
	char y;
} Pixel;

Pixel snake[100];
Pixel food;
int length;
char direction;


void placeSnakeFood() {
	
	int i = 0, j, k, x, y;
	char x1, x2, y1, y2;
	
	while(i < length) {
		
		/*place snake*/
		x1 = snake[i].x;
		y1 = snake[i].y;
		x2 = snake[i].x + DIM;
		y2 = snake[i].y + DIM;
			
		for(j = x1; j < x2; j++) {
			for(k = y1; k < y2; k++) {
				lcd_setpixel(j, k);
			}
		}
		i++;
			
	}
	
		/*place food*/
		x1 = food.x;
		y1 = food.y;
		x2 = food.x + DIM;
		y2 = food.y + DIM;
		
		for(x = x1; x < x2; x++) {
			for(y = y1; y < y2; y++) {
				lcd_setpixel(x, y);
			}
		}
	
}

void drawWalls(int resX, int resY) {
		int i = 0, j = 0;
		while (i < resY) {
			
			lcd_setpixel(0, i);
			lcd_setpixel(resX - 1, i);
			i++;
		}
		
		while (j < resX) {
			
			lcd_setpixel(j, 0);
			lcd_setpixel(j, resY - 1);
			j++;
		}
}
void drawMap(void) {

	
	int i = 0, j, k;
	char x1, x2, y1, y2;
	
	placeSnakeFood();
	
	if (mod == HARD) {
		/*walls*/
		drawWalls(int LCD_X_RES, int LCD_Y_RES);
	}
}


void init(void){

	int i;
	length = INIT_LEN;
	direction = RIGHT;
	
	snake[0].x = INIT_X;
	snake[0].y = INIT_Y;
	food.x = rand()%(LCD_X_RES - DIM);
	food.y = rand()%(LCD_Y_RES - DIM);
	
	i = 1;
	while(i < length){
		
		/*when going to right, increasing left dimm*/
		snake[i].x = snake[0].x - i * DIM;
		snake[i].y = snake[0].y;
		i++;
	}

}


void movement(char dir) {

	int i = length - 1;
	
	/*rest of body following the head*/
	while (i > 0) {
		snake[i].x = snake[i - 1].x;
		snake[i].y = snake[i - 1].y;
		i--;
	}
	
	int poz_y = LCD_Y_RES - DIM;
	int poz_x = LCD_X_RES - DIM;
	
	switch (dir)
	{
		case UP:
				if (snake[0].y <= 0) {
					snake[0].y = poz_y;
					} else {
					snake[0].y -= DIM;
				}
				break;
				
		case DOWN:
				if (snake[0].y >= poz_y) {
					snake[0].y = 0;
				} else {
					snake[0].y += DIM;
				}
				break;
				
		case LEFT:
				if (snake[0].x <= 0) {
					snake[0].x = poz_x;
				} else {
					snake[0].x -= DIM;
				}
				break;
			
		case RIGHT:
				if (snake[0].x >= poz_x) {
					snake[0].x = 0;
				} else {
					snake[0].x += DIM;
				}
				break;
						
	}
	
}


void selectMode(void) {
	

	lcd_clear();
	lcd_update();
	
	lcd_gotoXY(25, 0);
	lcd_str("Menu", 4);
	lcd_gotoXY(2, 1);
	lcd_str("Choose Mode", 11);
	
	lcd_gotoXY(10, 3);
	if (mod == EASY)
		lcd_str("<3 EASY", 7);
	else
		lcd_str("  EASY", 6);
	
	lcd_gotoXY(10, 4);
	if (mod == HARD)
		lcd_str("<3 HARD", 7);
	else
		lcd_str("  HARD", 6);
	
	while(1) {
		
		
		// mode selected
		if (!(PINA & (1 << PA0)))
		break;
		
		// DOWN
		if (!(PINA & (1 << PA4))) {
			mod = (mod + 1) % 2;
			lcd_clear();
			lcd_update();
			
			lcd_gotoXY(25, 0);
			lcd_str("Menu", 4);
			lcd_gotoXY(2, 1);
			lcd_str("Choose Mode", 11);
			
			lcd_gotoXY(10, 3);
			if (mod == EASY)
				lcd_str("<3 Easy", 7);
			else
				lcd_str("  EASY", 6);
			
			lcd_gotoXY(10, 4);
			if (mod == HARD)
				lcd_str("<3 HARD", 7);
			else
				lcd_str("  HARD", 6);
			
		}
		
		// UP
		if (!(PINA & (1 << PA3))) {
			mod = mod - 1;
			if (mod == -1)
			mod = 1;
			lcd_clear();
			lcd_update();
			
			lcd_gotoXY(25, 0);
			lcd_str("Menu", 4);
			lcd_gotoXY(2, 1);
			lcd_str("Choose Mode", 11);
			
			lcd_gotoXY(10, 3);
			if (mod == EASY)
				lcd_str("<3 EASY", 7);
			else
				lcd_str("  EASY", 6);
			
			lcd_gotoXY(10, 4);
			if (mod == HARD)
				lcd_str("<3 HARD", 7);
			else
				lcd_str("  HARD", 6);
			
		}
		
		_delay_ms(5000);
		
	}
	
	points = 0;
	_delay_ms(5000);
	lcd_clear();
	lcd_update();
	init();
	
}


void game_over(void) {

	char numar[2];
	
	itoa(points, numar, 10);
	
	lcd_init();
	lcd_clear();
	lcd_update();
	
	lcd_gotoXY(12, 1);
	lcd_str("Game Over", 9);
	
	lcd_gotoXY(16, 3);
	lcd_str("Points: ", 8);
	
	lcd_gotoXY(61, 3);
	/*choosing display*/
	if (points >= 10)
		lcd_str(numar, 2);
	else
		lcd_str(numar, 1);
	
	while(1){
		
		/*back to menu*/
		if (!(PINA & (1 << PA0))) {
			
			selectMode();
			break;
		}
		
	}
	
}


void cornerCases(void) {

	int i;
	
	/*out of rezolution*/
	if (mod == HARD) {
		if ((snake[0].x <= 0) || (snake[0].x >= LCD_X_RES))
			game_over();
		else if ((snake[0].y <= 0) || (snake[0].y >= LCD_Y_RES))
			game_over();
		
	}
	
	if (mod == EASY) {
		for(i = 1; i < length; i++){
			if ((snake[0].x == snake[i].x) && (snake[0].y == snake[i].y)){
				lives--;
				if (lives > 0) {
					
					char no_lives[1];
					itoa(lives, no_lives, 10);
					lcd_gotoXY(12, 1);
					lcd_str("Lives:", 6);
					lcd_gotoXY(50, 3);
					lcd_str(no_lives, 1);
					_delay_ms(5000);
					init();
				}
				else {
					game_over();
				}
			}
		}
	}
	
	/*eating itself*/
	if (mod == HARD) {
		i = 1;
		while (i < length) {
			if ((snake[0].x == snake[i].x) && (snake[0].y == snake[i].y)) {
				game_over();
			}
			i++;
		}
	}
	
	if (((snake[0].x <= food.x + DIM) &&
		(snake[0].y <= food.y + DIM) &&
		(snake[0].x >= food.x) &&
		(snake[0].y >= food.y)) ||
	    ((snake[0].x + DIM <= food.x + DIM)
		&& (snake[0].y + DIM <= food.y + DIM) &&
		(snake[0].x + DIM >= food.x) &&
		(snake[0].y + DIM >= food.y))) {
			
		/*increasing*/
		snake[length].x = snake[length - 1].x;
		snake[length].y = snake[length - 1].y;
		length++;
		/*new food*/
		food.x = rand() % (LCD_X_RES - DIM);
		food.y = rand() % (LCD_Y_RES - DIM);
		/*increasing points*/
		points++;
	}
	
}

void doIt(){
	movement(direction);
	cornerCases();
	drawMap();
	_delay_ms(5000);
	lcd_update();
}


int main(void) {
	
	lcd_init();
	lcd_clear();
	lcd_update();
	
	DDRA &= ~(1 << PA4); // DOWN
	PORTA |= (1 << PA4);
	
	DDRA &= ~(1 << PA2); // RIGHT
	PORTA |= (1 << PA2);
	
	DDRA &= ~(1 << PA3); // UP
	PORTA |= (1 << PA3);
	
	DDRA &= ~(1 << PA1); // LEFT
	PORTA |= (1 << PA1);
	
	DDRA &= ~(1 << PA0); //SELECT
	PORTA |= (1 << PA0);
	

	selectMode();
	
	lcd_gotoXY(11, 3);
	lcd_str("Start", 5);
	_delay_ms(30000);
	
	if (mod == HARD) {
		while(1) {
			
			
			lcd_clear();
			
			
			if (!(PINA & (1 << PA3)) && (direction != DOWN))
				direction = UP;
			if (!(PINA & (1 << PA4)) && (direction != UP))
				direction = DOWN;
			if (!(PINA & (1 << PA1)) && (direction != RIGHT))
				direction = LEFT;
			if (!(PINA & (1 << PA2)) && (direction != LEFT))
				direction = RIGHT;
			
			
			if (!(PINA & (1 << PA0))) {
				/*back to menu*/
				
				selectMode();
				lcd_gotoXY(11, 3);
				lcd_str("Start", 5);
				continue;
			}
			
			doIt();

		}
		
	} else if (mod == EASY) {
		while (lives > 0) {
			
			lcd_clear();
			
			
			if (!(PINA & (1 << PA3)) && (direction != DOWN))
				direction = UP;
			if (!(PINA & (1 << PA4)) && (direction != UP))
				direction = DOWN;
			if (!(PINA & (1 << PA1)) && (direction != RIGHT))
				direction = LEFT;
			if (!(PINA & (1 << PA2)) && (direction != LEFT))
				direction = RIGHT;
			
			
			if (!(PINA & (1 << PA0))) {
				/*back to menu*/
				
				selectMode();
				continue;
			}
			
			doIt();
			
		}
		game_over();
	}
}






